@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+
 module Data.X509.File (
     readSignedObject,
     readKeyFile,
@@ -14,6 +16,9 @@ import Data.Maybe
 import Data.PEM (PEM, pemContent, pemName, pemParseLBS)
 import qualified Data.X509 as X509
 import Data.X509.Memory (pemToKey)
+#if !defined(mingw32_HOST_OS) && (MIN_VERSION_unix(2,8,0))
+import System.Posix.IO
+#endif
 
 newtype PEMError = PEMError {displayPEMError :: String}
     deriving (Show)
@@ -23,7 +28,13 @@ instance Exception PEMError where
 
 readPEMs :: FilePath -> IO [PEM]
 readPEMs filepath = do
+#if defined(mingw32_HOST_OS) || !(MIN_VERSION_unix(2,8,0))
     content <- L.readFile filepath
+#else
+    fd <- openFd filepath ReadOnly defaultFileFlags{cloexec = True}
+    h <- fdToHandle fd
+    content <- L.hGetContents h
+#endif
     either (throw . PEMError) pure $ pemParseLBS content
 
 -- | return all the signed objects in a file.
